@@ -1,5 +1,5 @@
 package Mojo::DOM::Role::Analyzer ;
-$Mojo::DOM::Role::Analyzer::VERSION = '0.008';
+$Mojo::DOM::Role::Analyzer::VERSION = '0.009';
 use strict;
 use warnings;
 use Role::Tiny;
@@ -15,8 +15,8 @@ sub element_count {
 
 sub parent_all {
   my $self = shift;
-  my $tag = shift || $self->tag;
-  carp 'Unable to determine tag' if !$tag;
+  my $tag = shift;
+  carp 'Unable to determine tag' unless $tag;
   my $p_count = $self->root->find($tag)->size;
 
   my $enclosing_tag = $self->root->at($tag);
@@ -34,17 +34,87 @@ sub parent_ptags {
   return $self->parent_all('p');
 }
 
-# determine if a tag A comes before or after tag B in the dom
-sub compare {
-  my ($sel1, $sel2);
+sub _get_selectors {
+  my ($s, $sel1, $sel2);
   if (!$_[2]) {
-    my $s = shift;
+    $s = shift;
     $sel1 = $s->selector;
-    $sel2 = $s->root->at($_[0])->selector;
+    if (ref $_[0]) {
+      $sel2 = $_[0]->selector;
+    } else {
+      $sel2 = $s->root->at($_[0])->selector;
+    }
   } else {
+    $s = $_[0];
     $sel1 = $_[1]->selector;
     $sel2 = $_[2]->selector;
   }
+  return ($s, $sel1, $sel2);
+}
+
+# traverses the DOM upward to find the closest tag node
+sub closest_up {
+  return _closest(@_, 'up');
+}
+
+sub closest_down {
+  return _closest(@_, 'down');
+}
+
+sub _closest {
+  my $s = shift;
+  my $sel = $s->selector;
+  use Log::Log4perl::Shortcuts qw(:all);
+  my $tag = shift;
+  my $dir = shift || 'up';
+  if ($dir ne 'up') {
+    $dir = 'down';
+  }
+
+  my $found;
+  if ($dir eq 'up') {
+    $found = $s->root->find($tag)->grep(sub { ($s cmp $_) > 0  } );
+  } else {
+    $found = $s->root->find($tag)->grep(sub { ($s cmp $_) < 0  } );
+  }
+
+  return 0 unless $found->size;
+
+  my $shortest_dist;
+  my @shortest_selectors;
+  foreach my $f ($found->each) {
+    my $key = $f->selector;
+    my $dist = $s->root->at($sel)->distance($f);
+    if (!$shortest_dist) {
+      $shortest_dist = $dist;
+      push @shortest_selectors, $key;
+    } elsif ($dist <= $shortest_dist) {
+      if ($dist < $shortest_dist) {
+        @shortest_selectors = ();
+        push @shortest_selectors, $key;
+      } else {
+        $shortest_dist = $dist;
+        push @shortest_selectors, $key;
+      }
+    }
+  }
+
+  if (@shortest_selectors == 1) {
+    return $s->root->at($shortest_selectors[0]);
+  }
+
+  my @sorted = sort { $s->root->at($a) cmp $s->root->at($b) } @shortest_selectors;
+  if ($dir eq 'up') {
+    return $s->root->at($sorted[-1]);  # get furthers from the top (closest to node of interest)
+  } else {
+    return $s->root->at($sorted[0]);   # get futherst from the bottom (closest to node of interest)
+  }
+
+}
+
+# determine if a tag A comes before or after tag B in the dom
+sub compare {
+  my ($s, $sel1, $sel2) = _get_selectors(@_);
 
   my @t1_path = split / > /, $sel1;
   my @t2_path = split / > /, $sel2;
@@ -60,6 +130,16 @@ sub compare {
   }
 }
 
+sub distance {
+  my ($s, $sel1, $sel2) = _get_selectors(@_);
+
+  my $common = common($s, $s->root->at($sel1), $s->root->at($sel2));
+  my $dist_leg1 = $s->root->at($sel1)->depth - $common->depth;
+  my $dist_leg2 = $s->root->at($sel2)->depth - $common->depth;
+
+  return $dist_leg1 + $dist_leg2;
+}
+
 sub depth {
   my $s = shift;
   my $sel = $s->selector;
@@ -77,6 +157,33 @@ sub deepest {
   return $deepest_depth;
 }
 
+# find the common ancestor between two nodes
+sub common {
+  my ($s, $sel1, $sel2);
+  if ($_[1] && $_[2] && !ref $_[1] && !ref $_[2]) {
+    ($s, $sel1, $sel2) = @_;
+  } else {
+    ($s, $sel1, $sel2) = _get_selectors(@_);
+  }
+
+  my @t1_path = split / > /, $sel1;
+  my @t2_path = split / > /, $sel2;
+
+  my @last_common;
+  foreach my $p1 (@t1_path) {
+    my $p2 = shift(@t2_path);
+    if ($p1 eq $p2) {
+      push @last_common, $p1;
+    } else {
+      last;
+    }
+  }
+  my $common_selector = join ' > ', @last_common;
+
+  return $s->root->at($common_selector);
+
+}
+
 
 1; # Magic true value
 # ABSTRACT: miscellaneous methods for analyzing a DOM
@@ -122,7 +229,7 @@ Mojo::DOM::Role::Analyzer - miscellaneous methods for analyzing a DOM
 
 =head1 DESCRIPTION
 
-=head3 Operators
+=head2 Operators
 
 =head3 cmp
 
@@ -133,6 +240,32 @@ the dom. See C<compare> method below for return values.
 
 =head2 Methods
 
+=head3 closest_up
+
+  my $closest_up_dom = $dom->at('p')->closest_up('h1');
+
+Returns the node closest to the tag node of interest by searching upward through the DOM.
+
+=head4 closest_down
+
+  my $closest_down_dom = $dom->at('h1')->closest_down('p');
+
+Returns the node closest to the tag node of interest by searching downward through the DOM.
+
+=head3 closest_down
+
+=head3 distance
+
+=head4 C<$dom-E<gt>at($selector)-E<gt>distance($selector)>
+
+=head4 C<$dom-E<gt>at($selector)-E<gt>distance($dom)>
+
+=head4 C<$dom-E<gt>distance($dom1, $dom2)>
+
+Finds the distance between two nodes. The value is calculated by finding the
+lowest common ancestor node for the two nodes and then adding the distance from
+each individual node to the lowest common ancestor node.
+
 =head3 element_count
 
   $count = $dom->element_count;
@@ -156,7 +289,39 @@ that wraps all the tags indicated in the argument.
 A conveniece method that works like the C<parent_all> method but automatically supplies a
 C<'p'> tag argument for you.
 
-=head3 compare($dom1, $dom2)
+=head3 common
+
+=head4 C<$dom-E<gt>at($tag1)-E<gt>common($tag2)>
+
+=head4 C<$dom-E<gt>common($dom1, $dom2)>
+
+=head4 C<$dom-E<gt>common($selector_str1, $selector_str2)>
+
+  my $common_dom = $dom->at('div.bar')->common('div.foo');    # 'div.foo' is relative to root
+
+  # OR
+
+  my $dom1 = $dom->at('div.bar');
+  my $dom2 = $dom->at('div.foo');
+  my $common = $dom->common($dom1, $dom2);
+
+  # OR
+
+  my $common = $dom->common($dom->at('p')->selector, $dom->at('h1')->selector);
+
+Returns the lowest common ancestor node between two tag nodes or two selector strings.
+
+=head3 compare
+
+=head4 C<$dom-E<gt>at($tag1)-E<gt>compare($tag2)>
+
+=head4 C<compare($dom1, $dom2)>
+
+=head4 C<$dom1 cmp $dom2>
+
+  $dom->at('p.first')->compare('p.last');    # 'p.last' is relative to root
+
+  # OR
 
   my $dom1 = $dom->at('p.first');
   my $dom2 = $dom->at('p.last');
@@ -166,19 +331,15 @@ C<'p'> tag argument for you.
 
   my $result = $dom1 cmp $dom2;
 
-  # OR
-
-  $dom->at('p.first')->compare('p.last');    # 'p.last' is relative to root
-
 Compares the selectors of two $dom objects to see which comes first in the DOM.
 
 =over 1
 
-=item Returns a value of '-1' if the first argument comes before (is less than) the second.
+=item * Returns a value of '-1' if the first argument comes before (is less than) the second.
 
-=item Returns a value of '0' if the first and second arguments are the same.
+=item * Returns a value of '0' if the first and second arguments are the same.
 
-=item Returns a value of '1' if the first argument comes after (is greater than) the second.
+=item * Returns a value of '1' if the first argument comes after (is greater than) the second.
 
 =back
 
@@ -196,7 +357,7 @@ Finds the deeepest nested level within a node.
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
